local _, AT = ...

local Utils = AT.Utils

local BUFF_INFO = AT.Enums.BUFF_INFO
local TRANSPARENCY_STATE = AT.Enums.TRANSPARENCY_STATE

local FrameMouseover = {}
setmetatable(FrameMouseover, {__index = AT.FrameBase})
FrameMouseover.frame = nil
FrameMouseover.allowedTransparencyStates = {
  TRANSPARENCY_STATE.NO_MANA,
  TRANSPARENCY_STATE.NOT_IN_RANGE
}

AT.FrameMouseover = FrameMouseover

function FrameMouseover:construct(aId)
  obj = {}
  setmetatable(obj, {__index = self})
  obj:postConstruct(aId)
  obj:Init()
  return obj
end

function FrameMouseover:Init()
  self.frame = CreateFrame('FRAME', nil, UIParent)

  self.frame.text = self.frame:CreateFontString(nil, 'BACKGROUND', GameFontNormal)
  self.frame.text:SetPoint('TOPLEFT', self.padding, self.padding * -1)
  self.frame.text:SetJustifyV('TOP')
  self.frame.text:SetJustifyH('LEFT')
  self.frame.text:SetFont('Fonts\\FRIZQT__.TTF', 10, 'OUTLINE')
  self.frame.text:SetNonSpaceWrap(false)

  self:PostInit()
  self:Hide()
end

function FrameMouseover:UpdateSettings()
  AT.FrameBase.UpdateSettings(self, { ignorePoint = true })
end

function FrameMouseover:OnUpdate()
  if self.isVisible then
    local x, y = GetCursorPosition()
    local oX, oY = self:GetOffset()
    local scale = self.frame:GetEffectiveScale()
    self.frame:ClearAllPoints()
    self.frame:SetPoint('TOPLEFT', UIParent, 'BOTTOMLEFT', (x / scale) + oX, (y / scale) + oY)

    self.frame:Show()
  end
end

function FrameMouseover:Set(aStealableBuffsInfo)
  if not self:GetEnabled() then
    return
  end

  local text = ''

  for _, buffInfo in ipairs(aStealableBuffsInfo) do
    if buffInfo[BUFF_INFO.SHOW_IN_MOUSEOVER] then
      if not Utils:StrIsNilOrEmpty(text) then
        text = text..'\n'
      end
      text = string.format('%s|Hspell%s|h|T%s:14|t|c%s %s|h',
        text, buffInfo[BUFF_INFO.ID], buffInfo[BUFF_INFO.ICON], self.textColors.buffName, buffInfo[BUFF_INFO.NAME])
      if buffInfo[BUFF_INFO.EXPIRATION_TIME] ~= nil and buffInfo[BUFF_INFO.EXPIRATION_TIME] > 0 then
        text = string.format('%s|c%s - %s|r', text, self.textColors.buffExpTime, Utils:FormatExpirationTime(buffInfo[BUFF_INFO.EXPIRATION_TIME]))
      end
    end 
  end

  self.frame.text:SetText(text)
  self.frame:SetSize(self.frame.text:GetStringWidth() + self.padding * 2, self.frame.text:GetStringHeight() + self.padding * 2)

  self.isVisible = true
  self:OnUpdate()
  self.frame:Show()
end

function FrameMouseover:Hide()
  self.isVisible = false
  self.frame:Hide()
end
